﻿





''' <summary>
'''  Esta clase Base que define un eslabón. Es la clase [Handler] del diagrama
''' </summary>
Friend MustInherit Class EslabonCadenaConcreto
    Implements IDisposable


#Region "Propiedades y variables de la clase"

    ''' <summary>
    ''' Referencia al siguiente eslabón de la cadena
    ''' </summary>
    Public Property SiguienteEslabon As EslabonCadenaConcreto = Nothing

    ''' <summary>
    ''' Los datos de trabajo que circulan entre eslabones.
    ''' </summary>
    Public Property DatosTrabajoMensaje As MensajeVo = Nothing

#End Region


#Region "Constructores"

    ''' <summary>
    '''  Constructor estándar. Inicializa una nueva instancia de la clase 
    ''' </summary>
    Public Sub New()
        '
    End Sub

    ''' <summary>
    '''  Constructor estándar. Inicializa una nueva instancia de la clase 
    '''  y define quien es el nodo hijo
    ''' </summary>
    Public Sub New(ByVal paramSiguienteEslabon As EslabonCadenaConcreto)
        Me.SiguienteEslabon = paramSiguienteEslabon
    End Sub

#End Region

    ''' <summary>
    '''   Permite establecer el siguiente eslabón de la cadena
    ''' </summary>
    ''' <param name="paramSiguienteEslabon">
    '''   Un objeto EslabonCadenaConcreto que es el siguiente nodo de la cadena, el nodo hijo
    ''' </param>
    Public Sub EstablecerSiguienteEslabon(ByVal paramSiguienteEslabon As EslabonCadenaConcreto)
        Me.SiguienteEslabon = paramSiguienteEslabon
    End Sub

#Region "Miembros [MustOverride] de la clase "

    ''' <summary>
    '''  La clave del nodo.
    '''  Debe ser declarada por cada nodo hijo, por cada eslabón
    ''' </summary>
    Public MustOverride ReadOnly Property Key As String

    ''' <summary>
    '''  La función que hace el trabajo
    '''  Debe ser declarada por cada nodo hijo, por cada eslabón
    ''' </summary>
    ''' <param name="paramObjUnMensaje">
    '''   El objeto [MensajeVo] que contiene los datos iniciales para hacer el trabajo
    '''   Se pasa [ByRef] porque tiene que visitar varios objetos y así evito que se hagan copias inútiles del mismo  
    ''' </param>
    ''' <returns>Un objeto [RespuestaVo] que contiene la respuesta a la tarea encargada </returns>
    Public MustOverride Function ExecuteTarea(ByRef paramObjUnMensaje As MensajeVo) As RespuestaVo

#End Region



#Region "IDisposable Support.   [Para clases normales y bases ] Implementación de .NET Version: [2020/04/29]"

    ''' <summary>
    '''  Variable local para detectar llamadas redundantes que indica [True] el objeto se ha eliminado, [False], en caso contrario. 
    ''' </summary>
    Private campoDisposedValue As System.Boolean

    ''' <summary>
    '''     Obtiene un valor que indica si el objeto se ha eliminado. 
    ''' </summary>
    ''' <value>
    '''   <para>Tipo: <see cref="System.Boolean">Boolean</see></para>
    '''   <para>Es true si objeto se ha eliminado; en caso contrario, es false.</para>
    ''' </value>
    Public Overloads ReadOnly Property IsDisposed() As System.Boolean
        Get
            Return campoDisposedValue
        End Get
    End Property


    ''' <summary>
    '''  [Dispose] Código de limpieza de la clase
    ''' </summary>
    ''' <param name="paramQuienMeLLama">[True], llama el usuario, [False], llama el sistema</param>
    ''' <remarks>
    '''  <para>
    '''     Dispose (bool quienMeLLama) se ejecuta en dos escenarios distintos.
    '''  </para>
    '''  <para>
    '''     Si [quienMeLLama] es igual a true, el método ha sido llamado 
    '''     directamente, o indirectamente, por el código de un usuario. 
    '''     Los recursos administrados y no administrados se pueden eliminar.
    '''  </para>
    '''  <para>
    '''     Si [quienMeLLama] es igual a false, el método ha sido llamado por el
    '''     [runtime] desde el interior del finalizador y usted no debe hacer 
    '''     referencia a otros objetos, ni siquiera para eliminarlos.
    '''     Solo los recursos no administrados pueden ser eliminados.
    '''  </para>
    ''' </remarks>
    Protected Overridable Sub Dispose(paramQuienMeLLama As System.Boolean)
        If Me.campoDisposedValue = False Then
            Me.campoDisposedValue = True
            If paramQuienMeLLama Then
                '------------------------------------------------------------------------------------------
                ' 1) Desechar objetos SI administrados.
                If Not (SiguienteEslabon Is Nothing) Then
                    SiguienteEslabon = Nothing
                End If
                If Not (DatosTrabajoMensaje Is Nothing) Then
                    DatosTrabajoMensaje = Nothing
                End If

            End If
            '------------------------------------------------------------------------------------------
            ' 2) Liberar objetos NO administrados.
            '------------------------------------------------------------------------------------------
            ' 3) Establecer campos grandes como Null.
            '------------------------------------------------------------------------------------------
        End If
    End Sub


    ''' <summary>
    '''  [Dispose] Código de limpieza de la clase
    ''' </summary>
    ''' <remarks>
    '''  Visual Basic agregó este código para implementar correctamente el patrón descartable.
    '''  No cambie este código. Coloque el código de limpieza en Dispose(disposing As Boolean).
    ''' </remarks>
    Public Sub Dispose() Implements IDisposable.Dispose
        Me.Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub

#End Region



End Class
